@@ -1,8 +1,8 @@
- // -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
+// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 //
 //  RcppAnnoy -- Rcpp bindings to Annoy library for Approximate Nearest Neighbours
 //
-//  Copyright (C) 2014 - 2015  Dirk Eddelbuettel
+//  Copyright (C) 2014 - 2016  Dirk Eddelbuettel
 //
 //  This file is part of RcppAnnoy
 //
@@ -39,16 +39,99 @@
 #include "annoylib.h"
 #include "kissrandom.h"
 
+template< typename S, typename T, typename Distance, typename Random >
+class Annoy
+{
+protected:
+    AnnoyIndex<S, T, Distance, Random> *ptr;
+    unsigned int vectorsz;
+public:
+    Annoy(int n) : vectorsz(n) {
+        ptr = new AnnoyIndex<S, T, Distance, Random>(n);
+    }
+    ~Annoy() { if (ptr != NULL) delete ptr; }
+    void addItem(int32_t item, Rcpp::NumericVector dv) {
+        if (item < 0) Rcpp::stop("Inadmissible item value %d", item);
+        std::vector<float> fv(dv.size());
+        std::copy(dv.begin(), dv.end(), fv.begin());
+        ptr->add_item(item, &fv[0]);
+    }
+    void   callBuild(int n)               { ptr->build(n);                  }
+    void   callSave(std::string filename) { ptr->save(filename.c_str());    }
+    void   callLoad(std::string filename) { ptr->load(filename.c_str());    }
+    void   callUnload()                   { ptr->unload();                  }
+    int    getNItems()                    { return ptr->get_n_items();      }
+    double getDistance(int i, int j)      { return ptr->get_distance(i, j); }
+    void   verbose(bool v)                { ptr->verbose(v);                }
+
+    std::vector<int32_t> getNNsByItem(int32_t item, size_t n) {
+        std::vector<int32_t> result;
+        ptr->get_nns_by_item(item, n, -1, &result, NULL);
+        return result;
+    }
+
+    Rcpp::List getNNsByItemList(S item, size_t n,
+                                size_t search_k, bool include_distances) {
+        if (include_distances) {
+            std::vector<S> result;
+            std::vector<T> distances;
+            ptr->get_nns_by_item(item, n, search_k, &result, &distances);
+            return Rcpp::List::create(Rcpp::Named("item")     = result,
+                                      Rcpp::Named("distance") = distances);
+        } else {
+            std::vector<S> result;
+            ptr->get_nns_by_item(item, n, search_k, &result, NULL);
+            return Rcpp::List::create(Rcpp::Named("item") = result);
+        }
+    }
+
+    std::vector<int32_t> getNNsByVector(std::vector<double> dv, size_t n) {
+        std::vector<float> fv(dv.size());
+        std::copy(dv.begin(), dv.end(), fv.begin());
+        std::vector<int32_t> result;
+        ptr->get_nns_by_vector(&fv[0], n, -1, &result, NULL);
+        return result;
+    }
+    
+    Rcpp::List getNNsByVectorList(std::vector<T> fv, size_t n,
+                                  size_t search_k, bool include_distances) {
+        if (fv.size() != vectorsz) {
+            Rcpp::stop("fv.size() != vector_size");
+        }
+        if (include_distances) {
+            std::vector<S> result;
+            std::vector<T> distances;
+            ptr->get_nns_by_vector(&fv[0], n, search_k, &result, &distances);
+            return Rcpp::List::create(
+                Rcpp::Named("item") = result,
+                Rcpp::Named("distance") = distances);
+        } else {
+            std::vector<S> result;
+            ptr->get_nns_by_vector(&fv[0], n, search_k, &result, NULL);
+            return Rcpp::List::create(Rcpp::Named("item") = result);
+        }
+    }
+    
+    std::vector<double> getItemsVector(int32_t item) {
+        std::vector<float> fv;
+        ptr->get_item(item, &fv);
+        std::vector<double> dv(fv.size());
+        std::copy(fv.begin(), fv.end(), dv.begin());
+        return dv;
+    }
+
+};
+
 // this breaks Rcpp Modules as we can not have nested (ie two-level) templates :-/
-//typedef Annoy<int32_t, float, Angular<int32_t, float, Kiss64Random> >   AnnoyAngular;
-//typedef Annoy<int32_t, float, Euclidean<int32_t, float, Kiss64Random> > AnnoyEuclidean;
+typedef Annoy<int32_t, float, Angular, Kiss64Random>   AnnoyAngular;
+typedef Annoy<int32_t, float, Euclidean, Kiss64Random> AnnoyEuclidean;
 
 // this does too
 //typedef Angular<int32_t, float, Kiss64Random>     AnnoyAngularDist;
 //typedef Euclidean<int32_t, float, Kiss64Random>   AnnoyEucledianDist;
 //typedef Annoy<int32_t, float, AnnoyAngularDist>   AnnoyAngular;
 //typedef Annoy<int32_t, float, AnnoyEucledianDist> AnnoyEuclidean;
-
+/*
 class AnnoyBase {
 protected:
     AnnoyIndexInterface<int32_t, float> *ptr;
@@ -132,7 +215,7 @@ public:
                                                               { return AnnoyBase::getNNsByVector(dv, n); }
     inline std::vector<double> getItemsVector(int32_t item)   { return AnnoyBase::getItemsVector(item);  }
 };
-
+*/
 
 
 
@@ -148,8 +231,14 @@ RCPP_MODULE(AnnoyAngular) {
         .method("load",           &AnnoyAngular::callLoad,        "load index from file")
         .method("unload",         &AnnoyAngular::callUnload,      "unload index")
         .method("getDistance",    &AnnoyAngular::getDistance,     "get distance between i and j")
-        .method("getNNsByItem",   &AnnoyAngular::getNNsByItem,    "retrieve Nearest Neigbours given item")
-        .method("getNNsByVector", &AnnoyAngular::getNNsByVector,  "retrieve Nearest Neigbours given vector")
+        .method("getNNsByItem",   &AnnoyAngular::getNNsByItem,
+                "retrieve Nearest Neigbours given item")
+        .method("getNNsByItemList",  &AnnoyAngular::getNNsByItemList,
+                "retrieve Nearest Neigbours given item")
+        .method("getNNsByVector", &AnnoyAngular::getNNsByVector,
+                "retrieve Nearest Neigbours given vector")
+        .method("getNNsByVectorList",  &AnnoyAngular::getNNsByVectorList,
+                "retrieve Nearest Neigbours given vector")
         .method("getItemsVector", &AnnoyAngular::getItemsVector,  "retrieve item vector")
         .method("getNItems",      &AnnoyAngular::getNItems,       "get N items")
         .method("setVerbose",     &AnnoyAngular::verbose,         "set verbose")
@@ -168,8 +257,14 @@ RCPP_MODULE(AnnoyEuclidean) {
         .method("load",           &AnnoyEuclidean::callLoad,       "load index from file")
         .method("unload",         &AnnoyEuclidean::callUnload,     "unload index")
         .method("getDistance",    &AnnoyEuclidean::getDistance,    "get distance between i and j")
-        .method("getNNsByItem",   &AnnoyEuclidean::getNNsByItem,   "retrieve Nearest Neigbours given item")
-        .method("getNNsByVector", &AnnoyEuclidean::getNNsByVector, "retrieve Nearest Neigbours given vector")
+        .method("getNNsByItem",   &AnnoyEuclidean::getNNsByItem,
+                "retrieve Nearest Neigbours given item")
+        .method("getNNsByItemList",  &AnnoyEuclidean::getNNsByItemList,
+                "retrieve Nearest Neigbours given item")
+        .method("getNNsByVector", &AnnoyEuclidean::getNNsByVector,
+                "retrieve Nearest Neigbours given vector")
+        .method("getNNsByVectorList",&AnnoyEuclidean::getNNsByVectorList,
+                "retrieve Nearest Neigbours given vector")
         .method("getItemsVector", &AnnoyEuclidean::getItemsVector, "retrieve item vector")
         .method("getNItems",      &AnnoyEuclidean::getNItems,      "get N items")
         .method("setVerbose",     &AnnoyEuclidean::verbose,        "set verbose")
